@@ -10,13 +10,13 @@ lazy_static::lazy_static! {
     static ref SENDER: Cert =
         Cert::from_bytes(&include_bytes!("../tests/data/keys/sender.pgp")[..])
         .unwrap();
-    static ref ZEROS_1_MB: Vec<u8> = vec![0; 1 * 1024 * 1024];
+    static ref ZEROS_1_MB: Vec<u8> = vec![0; 1024 * 1024];
     static ref ZEROS_10_MB: Vec<u8> = vec![0; 10 * 1024 * 1024];
 }
 
 fn verify(bytes: &[u8], sender: &Cert) {
     let mut sink = Vec::new();
-    decrypt::verify(&mut sink, &bytes, sender).unwrap();
+    decrypt::verify(&mut sink, bytes, sender).unwrap();
 }
 
 fn bench_verify(c: &mut Criterion) {
@@ -34,7 +34,7 @@ fn bench_verify(c: &mut Criterion) {
             group.bench_with_input(
                 BenchmarkId::new("verify", m.len()),
                 &signed,
-                |b, s| b.iter(|| verify(&s, &SENDER)),
+                |b, s| b.iter(|| verify(s, &SENDER)),
             );
         });
 
