@@ -275,6 +275,7 @@ pub struct CertBuilder<'a> {
     user_attributes: Vec<(Option<SignatureBuilder>, packet::UserAttribute)>,
     password: Option<Password>,
     revocation_keys: Option<Vec<RevocationKey>>,
+    exportable: bool,
     phantom: PhantomData<&'a ()>,
 }
 assert_send_and_sync!(CertBuilder<'_>);
@@ -331,6 +332,7 @@ impl CertBuilder<'_> {
             user_attributes: vec![],
             password: None,
             revocation_keys: None,
+            exportable: true,
             phantom: PhantomData,
         }
     }
@@ -507,6 +509,49 @@ impl CertBuilder<'_> {
         self
     }
 
+    /// Sets whether the certificate is exportable.
+    ///
+    /// This method controls whether the certificate is exportable.
+    /// If the certificate builder is configured to make a
+    /// non-exportable certificate, then all of the signatures that it
+    /// creates include the an [Exportable Certification] subpacket
+    /// that is set to `false`.
+    ///
+    /// [Exportable Certification]: https://datatracker.ietf.org/doc/html/rfc4880#section-5.2.3.11
+    ///
+    /// # Examples
+    ///
+    /// When exporting a non-exportable certificate, nothing will be
+    /// exported.  This is also the case when the output is ASCII
+    /// armored.
+    ///
+    /// ```
+    /// use sequoia_openpgp as openpgp;
+    /// use openpgp::Result;
+    /// use openpgp::cert::prelude::*;
+    /// use openpgp::parse::Parse;
+    /// use openpgp::serialize::Serialize;
+    ///
+    /// # fn main() -> openpgp::Result<()> {
+    /// let (cert, _) =
+    ///     CertBuilder::general_purpose(None, Some("alice@example.org"))
+    ///         .set_exportable(false)
+    ///         .generate()?;
+    /// let mut exported = Vec::new();
+    /// cert.armored().export(&mut exported)?;
+    ///
+    /// let certs = CertParser::from_bytes(&exported)?
+    ///     .collect::<Result<Vec<Cert>>>()?;
+    /// assert_eq!(certs.len(), 0);
+    /// assert_eq!(exported.len(), 0, "{}", String::from_utf8_lossy(&exported));
+    /// # Ok(())
+    /// # }
+    /// ```
+    pub fn set_exportable(mut self, exportable: bool) -> Self {
+        self.exportable = exportable;
+        self
+    }
+
     /// Adds a User ID.
     ///
     /// Adds a User ID to the certificate.  The first User ID that is
@@ -1352,7 +1397,9 @@ impl CertBuilder<'_> {
         // Generate & self-sign primary key.
         let (primary, sig, mut signer) = self.primary_key(creation_time)?;
 
-        let mut cert = Cert::try_from(vec![
+        // Construct a skeleton cert.  We need this to bind the new
+        // components to.
+        let cert = Cert::try_from(vec![
             Packet::SecretKey({
                 let mut primary = primary.clone();
                 if let Some(ref password) = self.password {
@@ -1360,8 +1407,13 @@ impl CertBuilder<'_> {
                 }
                 primary
             }),
-            sig.into(),
         ])?;
+        // We will, however, collect any signatures and components in
+        // a separate vector, and only add them in the end, so that we
+        // canonicalize the new certificate just once.
+        let mut acc = vec![
+            Packet::from(sig),
+        ];
 
         // We want to mark exactly one User ID or Attribute as primary.
         // First, figure out whether one of the binding signature
@@ -1380,7 +1432,8 @@ impl CertBuilder<'_> {
         for (template, uid) in self.userids.into_iter() {
             let sig = template.unwrap_or_else(
                 || SignatureBuilder::new(SignatureType::PositiveCertification));
-            let sig = Self::signature_common(sig, creation_time)?;
+            let sig = Self::signature_common(sig, creation_time,
+                                             self.exportable)?;
             let mut sig = Self::add_primary_key_metadata(sig, &self.primary)?;
 
             // Make sure we mark exactly one User ID or Attribute as
@@ -1402,15 +1455,16 @@ impl CertBuilder<'_> {
             }
 
             let signature = uid.bind(&mut signer, &cert, sig)?;
-            cert = cert.insert_packets(
-                vec![Packet::from(uid), signature.into()])?;
+            acc.push(uid.into());
+            acc.push(signature.into());
         }
 
         // Sign UserAttributes.
         for (template, ua) in self.user_attributes.into_iter() {
             let sig = template.unwrap_or_else(
                 || SignatureBuilder::new(SignatureType::PositiveCertification));
-            let sig = Self::signature_common(sig, creation_time)?;
+            let sig = Self::signature_common(
+                sig, creation_time, self.exportable)?;
             let mut sig = Self::add_primary_key_metadata(sig, &self.primary)?;
 
             // Make sure we mark exactly one User ID or Attribute as
@@ -1432,8 +1486,8 @@ impl CertBuilder<'_> {
             }
 
             let signature = ua.bind(&mut signer, &cert, sig)?;
-            cert = cert.insert_packets(
-                vec![Packet::from(ua), signature.into()])?;
+            acc.push(ua.into());
+            acc.push(signature.into());
         }
 
         // Sign subkeys.
@@ -1446,12 +1500,15 @@ impl CertBuilder<'_> {
 
             let sig = template.unwrap_or_else(
                 || SignatureBuilder::new(SignatureType::SubkeyBinding));
-            let sig = Self::signature_common(sig, creation_time)?;
+            let sig = Self::signature_common(
+                sig, creation_time, self.exportable)?;
             let mut builder = sig
                 .set_key_flags(flags.clone())?
                 .set_key_validity_period(blueprint.validity.or(self.primary.validity))?;
 
-            if flags.for_certification() || flags.for_signing() {
+            if flags.for_certification() || flags.for_signing()
+                || flags.for_authentication()
+            {
                 // We need to create a primary key binding signature.
                 let mut subkey_signer = subkey.clone().into_keypair().unwrap();
                 let backsig =
@@ -1469,10 +1526,13 @@ impl CertBuilder<'_> {
             if let Some(ref password) = self.password {
                 subkey.secret_mut().encrypt_in_place(password)?;
             }
-            cert = cert.insert_packets(vec![Packet::SecretSubkey(subkey),
-                                           signature.into()])?;
+            acc.push(subkey.into());
+            acc.push(signature.into());
         }
 
+        // Now add the new components and canonicalize once.
+        let cert = cert.insert_packets(acc)?;
+
         let revocation = CertRevocationBuilder::new()
             .set_signature_creation_time(creation_time)?
             .set_reason_for_revocation(
@@ -1495,7 +1555,8 @@ impl CertBuilder<'_> {
             .generate_key(KeyFlags::empty().set_certification())?;
         key.set_creation_time(creation_time)?;
         let sig = SignatureBuilder::new(SignatureType::DirectKey);
-        let sig = Self::signature_common(sig, creation_time)?;
+        let sig = Self::signature_common(
+            sig, creation_time, self.exportable)?;
         let mut sig = Self::add_primary_key_metadata(sig, &self.primary)?;
 
         if let Some(ref revocation_keys) = self.revocation_keys {
@@ -1511,13 +1572,18 @@ impl CertBuilder<'_> {
 
     /// Common settings for generated signatures.
     fn signature_common(builder: SignatureBuilder,
-                        creation_time: time::SystemTime)
+                        creation_time: time::SystemTime,
+                        exportable: bool)
                         -> Result<SignatureBuilder>
     {
-        builder
+        let mut builder = builder
             // GnuPG wants at least a 512-bit hash for P521 keys.
             .set_hash_algo(HashAlgorithm::SHA512)
-            .set_signature_creation_time(creation_time)
+            .set_signature_creation_time(creation_time)?;
+        if ! exportable {
+            builder = builder.set_exportable_certification(false)?;
+        }
+        Ok(builder)
     }
 
 
@@ -1549,7 +1615,9 @@ mod tests {
     use crate::Fingerprint;
     use crate::packet::signature::subpacket::{SubpacketTag, SubpacketValue};
     use crate::types::PublicKeyAlgorithm;
+    use crate::parse::Parse;
     use crate::policy::StandardPolicy as P;
+    use crate::serialize::Serialize;
 
     #[test]
     fn all_opts() {
@@ -1592,7 +1660,7 @@ mod tests {
         let sig =
             cert.primary_key().with_policy(p, None).unwrap().binding_signature();
         assert_eq!(sig.typ(), crate::types::SignatureType::DirectKey);
-        assert!(sig.features().unwrap().supports_mdc());
+        assert!(sig.features().unwrap().supports_seipdv1());
     }
 
     #[test]
@@ -1625,7 +1693,7 @@ mod tests {
                    PublicKeyAlgorithm::EdDSA);
         assert!(cert1.subkeys().next().is_none());
         assert!(cert1.with_policy(p, None).unwrap().primary_userid().unwrap()
-                .binding_signature().features().unwrap().supports_mdc());
+                .binding_signature().features().unwrap().supports_seipdv1());
     }
 
     #[test]
@@ -1848,7 +1916,7 @@ mod tests {
     #[test]
     fn primary_user_things() -> Result<()> {
         fn count_primary_user_things(c: Cert) -> usize {
-            c.into_packets().map(|p| match p {
+            c.into_packets2().map(|p| match p {
                 Packet::Signature(s) if s.primary_userid().unwrap_or(false)
                     => 1,
                 _ => 0,
@@ -1929,4 +1997,112 @@ mod tests {
 
         Ok(())
     }
+
+    #[test]
+    fn non_exportable_cert() -> Result<()> {
+        // Make sure that when we export a non-exportable cert,
+        // nothing is exported.
+
+        let (cert, _) =
+            CertBuilder::general_purpose(None, Some("alice@example.org"))
+            .set_exportable(false)
+            .generate()?;
+
+        let (bob, _) =
+            CertBuilder::general_purpose(None, Some("bob@example.org"))
+            .generate()?;
+
+        // Have Bob certify Alice's primary User ID with an exportable
+        // signature.  This shouldn't make Alice's certificate
+        // exportable.
+        let mut keypair = bob.primary_key().key().clone()
+            .parts_into_secret()?.into_keypair()?;
+        let certification = cert.userids().nth(0).unwrap()
+            .certify(&mut keypair, &cert,
+                     SignatureType::PositiveCertification,
+                     None, None)?;
+        let cert = cert.insert_packets(certification)?;
+
+        macro_rules! check {
+            ($cert: expr, $export: ident, $expected: expr) => {
+                let mut exported = Vec::new();
+                $cert.$export(&mut exported)?;
+
+                let certs = CertParser::from_bytes(&exported)?
+                    .collect::<Result<Vec<Cert>>>()?;
+
+                assert_eq!(certs.len(), $expected);
+
+                if $expected == 0 {
+                    assert_eq!(exported.len(), 0,
+                               "{}", String::from_utf8_lossy(&exported));
+                } else {
+                    assert!(exported.len() > 0);
+                }
+            }
+        }
+
+        // Binary cert:
+        check!(cert, export, 0);
+        check!(cert, serialize, 1);
+
+        // Binary TSK:
+        check!(cert.as_tsk(), export, 0);
+        check!(cert.as_tsk(), serialize, 1);
+
+        // Armored cert:
+        check!(cert.armored(), export, 0);
+        check!(cert.armored(), serialize, 1);
+
+        // Armored TSK:
+        check!(cert.as_tsk().armored(), export, 0);
+        check!(cert.as_tsk().armored(), serialize, 1);
+
+        // Have Alice add a exportable self signature.  Now her's
+        // certificate should be exportable.
+        let mut keypair = cert.primary_key().key().clone()
+            .parts_into_secret()?.into_keypair()?;
+        let certification = cert.userids().nth(0).unwrap()
+            .certify(&mut keypair, &cert,
+                     SignatureType::PositiveCertification,
+                     None, None)?;
+        let cert = cert.insert_packets(certification)?;
+
+        macro_rules! check {
+            ($cert: expr, $export: ident, $expected: expr) => {
+                let mut exported = Vec::new();
+                $cert.$export(&mut exported)?;
+
+                let certs = CertParser::from_bytes(&exported)?
+                    .collect::<Result<Vec<Cert>>>()?;
+
+                assert_eq!(certs.len(), $expected);
+
+                if $expected == 0 {
+                    assert_eq!(exported.len(), 0,
+                               "{}", String::from_utf8_lossy(&exported));
+                } else {
+                    assert!(exported.len() > 0);
+                }
+            }
+        }
+
+        // Binary cert:
+        check!(cert, export, 1);
+        check!(cert, serialize, 1);
+
+        // Binary TSK:
+        check!(cert.as_tsk(), export, 1);
+        check!(cert.as_tsk(), serialize, 1);
+
+        // Armored cert:
+        check!(cert.armored(), export, 1);
+        check!(cert.armored(), serialize, 1);
+
+        // Armored TSK:
+        check!(cert.as_tsk().armored(), export, 1);
+        check!(cert.as_tsk().armored(), serialize, 1);
+
+        Ok(())
+    }
 }
