@@ -3,6 +3,7 @@ use std::mem;
 use std::vec;
 use std::path::Path;
 
+use buffered_reader::BufferedReader;
 use lalrpop_util::ParseError;
 
 use crate::{
@@ -11,6 +12,7 @@ use crate::{
     packet::Tag,
     Packet,
     parse::{
+        Cookie,
         Parse,
         PacketParserResult,
         PacketParser
@@ -224,7 +226,7 @@ impl KeyringValidator {
         } else {
             match r {
                 Ok(_) => KeyringValidity::KeyringPrefix,
-                Err(ParseError::UnrecognizedEOF { .. }) =>
+                Err(ParseError::UnrecognizedEof { .. }) =>
                     KeyringValidity::KeyringPrefix,
                 Err(err) =>
                     KeyringValidity::Error(
@@ -521,7 +523,7 @@ impl<'a> From<PacketParserResult<'a>> for CertParser<'a>
                     tracer!(TRACE, "PacketParserResult::next", 0);
                     if let Some(reader) = retry_with_reader.take() {
                         // Try to find the next (armored) blob.
-                        match PacketParser::from_buffered_reader(reader) {
+                        match PacketParser::from_cookie_reader(reader) {
                             Ok(PacketParserResult::Some(pp)) => {
                                 // We read at least one packet.  Try
                                 // to parse the next cert.
@@ -594,6 +596,14 @@ impl<'a> From<Vec<Packet>> for CertParser<'a> {
 
 impl<'a> Parse<'a, CertParser<'a>> for CertParser<'a>
 {
+    /// Initializes a `CertParser` from a `BufferedReader`.
+    fn from_buffered_reader<R>(reader: R) -> Result<CertParser<'a>>
+    where
+        R: BufferedReader<Cookie> + 'a,
+    {
+        Ok(Self::from(PacketParser::from_buffered_reader(reader)?))
+    }
+
     /// Initializes a `CertParser` from a `Read`er.
     fn from_reader<R: 'a + io::Read + Send + Sync>(reader: R) -> Result<Self> {
         Ok(Self::from(PacketParser::from_reader(reader)?))
@@ -873,21 +883,20 @@ impl<'a> CertParser<'a> {
             Some(cert)
         }).and_then(|mut cert| {
             let primary_fp: KeyHandle = cert.key_handle();
-            let primary_keyid = KeyHandle::KeyID(primary_fp.clone().into());
 
             // The parser puts all of the signatures on the
             // certifications field.  Split them now.
 
-            split_sigs(&primary_fp, &primary_keyid, &mut cert.primary);
+            split_sigs(&primary_fp, &mut cert.primary);
 
             for b in cert.userids.iter_mut() {
-                split_sigs(&primary_fp, &primary_keyid, b);
+                split_sigs(&primary_fp, b);
             }
             for b in cert.user_attributes.iter_mut() {
-                split_sigs(&primary_fp, &primary_keyid, b);
+                split_sigs(&primary_fp, b);
             }
             for b in cert.subkeys.iter_mut() {
-                split_sigs(&primary_fp, &primary_keyid, b);
+                split_sigs(&primary_fp, b);
             }
 
             let cert = cert.canonicalize();
@@ -913,7 +922,7 @@ impl<'a> CertParser<'a> {
 
 /// Splits the signatures in b.certifications into the correct
 /// vectors.
-pub(crate) fn split_sigs<C>(primary: &KeyHandle, primary_keyid: &KeyHandle,
+pub(crate) fn split_sigs<C>(primary: &KeyHandle,
                             b: &mut ComponentBundle<C>)
 {
     let mut self_signatures = Vec::with_capacity(0);
@@ -924,11 +933,8 @@ pub(crate) fn split_sigs<C>(primary: &KeyHandle, primary_keyid: &KeyHandle,
     for sig in mem::replace(&mut b.certifications, Vec::with_capacity(0)) {
         let typ = sig.typ();
 
-        let issuers =
-            sig.get_issuers();
-        let is_selfsig =
-            issuers.contains(primary)
-            || issuers.contains(primary_keyid);
+        let issuers = sig.get_issuers();
+        let is_selfsig = issuers.iter().any(|kh| kh.aliases(primary));
 
         use crate::SignatureType::*;
         if typ == KeyRevocation
@@ -1717,8 +1723,10 @@ mod test {
             .zip(certs_parsed.into_iter())
             .enumerate()
         {
-            let ps_orig: Vec<Packet> = c_orig.into_packets().collect();
-            let ps_parsed: Vec<Packet> = c_parsed.into_packets().collect();
+            let ps_orig: Vec<Packet> =
+                c_orig.as_tsk().into_packets().collect();
+            let ps_parsed: Vec<Packet> =
+                c_parsed.as_tsk().into_packets().collect();
             assert_eq!(ps_orig.len(), ps_parsed.len(),
                        "number of packets: expected vs. got");
 
@@ -1809,7 +1817,7 @@ mod test {
                 None, Some("a@example.org"))
             .generate()?;
         let cert_1_packets: Vec<Packet>
-            = cert_1.into_packets().collect();
+            = cert_1.into_packets2().collect();
 
         let (cert_2, _) =
             CertBuilder::general_purpose(
