@@ -54,9 +54,7 @@
 //!
 //!     if cert.fingerprint() == fpr {
 //!         // Found it!  Try to convert it to a Cert.
-//!         if let cert = Cert::try_from(cert) {
-//!             return cert;
-//!         }
+//!         return Cert::try_from(cert);
 //!     }
 //! }
 //!
@@ -459,8 +457,11 @@ impl<'a> Parse<'a, RawCert<'a>> for RawCert<'a> {
     /// Returns the first RawCert encountered in the reader.
     ///
     /// Returns an error if there are multiple certificates.
-    fn from_reader<R: 'a + Read + Send + Sync>(reader: R) -> Result<Self> {
-        let mut parser = RawCertParser::from_reader(reader)?;
+    fn from_buffered_reader<R>(reader: R) -> Result<RawCert<'a>>
+    where
+        R: BufferedReader<Cookie> + 'a
+    {
+        let mut parser = RawCertParser::from_buffered_reader(reader)?;
         if let Some(cert_result) = parser.next() {
             if parser.next().is_some() {
                 Err(Error::MalformedCert(
@@ -473,6 +474,14 @@ impl<'a> Parse<'a, RawCert<'a>> for RawCert<'a> {
             Err(Error::MalformedCert("No data".into()).into())
         }
     }
+
+    /// Returns the first RawCert encountered in the reader.
+    ///
+    /// Returns an error if there are multiple certificates.
+    fn from_reader<R: 'a + Read + Send + Sync>(reader: R) -> Result<Self> {
+        let br = Generic::with_cookie(reader, None, Cookie::default());
+        Self::from_buffered_reader(br)
+    }
 }
 
 impl<'a> crate::seal::Sealed for RawCert<'a> {}
@@ -614,12 +623,12 @@ impl<'a> RawCertParser<'a> {
         }
 
         // Strip the Dup reader.
-        let mut reader = dup.as_boxed().into_inner().expect("inner");
+        let mut reader = dup.into_boxed().into_inner().expect("inner");
 
         if dearmor {
-            reader = armor::Reader::from_buffered_reader(
+            reader = armor::Reader::from_cookie_reader(
                 reader, armor::ReaderMode::Tolerant(None),
-                Default::default()).as_boxed();
+                Default::default()).into_boxed();
 
             let mut dup = Dup::with_cookie(reader, Default::default());
             match Header::parse(&mut dup) {
@@ -639,7 +648,7 @@ impl<'a> RawCertParser<'a> {
                 }
             }
 
-            reader = dup.as_boxed().into_inner().expect("inner");
+            reader = dup.into_boxed().into_inner().expect("inner");
         }
 
         Ok(RawCertParser {
@@ -655,6 +664,14 @@ impl<'a> RawCertParser<'a> {
 
 impl<'a> Parse<'a, RawCertParser<'a>> for RawCertParser<'a>
 {
+    /// Initializes a `RawCertParser` from a `BufferedReader`.
+    fn from_buffered_reader<R>(reader: R) -> Result<RawCertParser<'a>>
+    where
+        R: BufferedReader<Cookie> + 'a
+    {
+        RawCertParser::new(reader)
+    }
+
     /// Initializes a `RawCertParser` from a `Read`er.
     fn from_reader<R: 'a + Read + Send + Sync>(reader: R) -> Result<Self> {
         RawCertParser::new(Generic::with_cookie(reader, None, Default::default()))
@@ -703,15 +720,15 @@ impl<'a> Iterator for RawCertParser<'a>
             // Get the reader,
             let reader = std::mem::replace(
                 &mut self.reader,
-                EOF::with_cookie(Default::default()).as_boxed());
+                EOF::with_cookie(Default::default()).into_boxed());
 
             // peel off the armor reader,
             let reader = reader.into_inner().expect("the armor reader");
 
             // and install a new one!
-            self.reader = armor::Reader::from_buffered_reader(
+            self.reader = armor::Reader::from_cookie_reader(
                 reader, armor::ReaderMode::Tolerant(None),
-                Default::default()).as_boxed();
+                Default::default()).into_boxed();
         }
 
         if self.reader.eof() {
@@ -1499,7 +1516,7 @@ mod test {
                 None, Some("a@example.org"))
             .generate()?;
         let cert_1_packets: Vec<Packet>
-            = cert_1.into_packets().collect();
+            = cert_1.into_packets2().collect();
 
         let (cert_2, _) =
             CertBuilder::general_purpose(
