@@ -19,6 +19,7 @@ use crate::packet::{
     UserAttribute,
     UserID,
 };
+use crate::packet::signature::subpacket::NotationDataFlags;
 use crate::cert::prelude::*;
 
 /// A builder for revocation certificates for OpenPGP certificates.
@@ -83,6 +84,7 @@ pub struct CertRevocationBuilder {
 }
 assert_send_and_sync!(CertRevocationBuilder);
 
+#[allow(clippy::new_without_default)]
 impl CertRevocationBuilder {
     /// Returns a new `CertRevocationBuilder`.
     ///
@@ -182,6 +184,84 @@ impl CertRevocationBuilder {
         })
     }
 
+    /// Adds a notation to the revocation certificate.
+    ///
+    /// Unlike the [`CertRevocationBuilder::set_notation`] method, this function
+    /// does not first remove any existing notation with the specified name.
+    ///
+    /// See [`SignatureBuilder::add_notation`] for further documentation.
+    ///
+    /// [`SignatureBuilder::add_notation`]: crate::packet::signature::SignatureBuilder::add_notation()
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use sequoia_openpgp as openpgp;
+    /// # use openpgp::Result;
+    /// use openpgp::cert::prelude::*;
+    /// use openpgp::packet::signature::subpacket::NotationDataFlags;
+    ///
+    /// # fn main() -> Result<()> {
+    /// let builder = CertRevocationBuilder::new().add_notation(
+    ///     "revocation-policy@example.org",
+    ///     "https://policy.example.org/cert-revocation-policy",
+    ///     NotationDataFlags::empty().set_human_readable(),
+    ///     false,
+    /// );
+    /// # Ok(())
+    /// # }
+    pub fn add_notation<N, V, F>(self, name: N, value: V, flags: F,
+                                 critical: bool)
+        -> Result<Self>
+    where
+        N: AsRef<str>,
+        V: AsRef<[u8]>,
+        F: Into<Option<NotationDataFlags>>,
+    {
+        Ok(Self {
+            builder: self.builder.add_notation(name, value, flags, critical)?
+        })
+    }
+
+    /// Sets a notation to the revocation certificate.
+    ///
+    /// Unlike the [`CertRevocationBuilder::add_notation`] method, this function
+    /// first removes any existing notation with the specified name.
+    ///
+    /// See [`SignatureBuilder::set_notation`] for further documentation.
+    ///
+    /// [`SignatureBuilder::set_notation`]: crate::packet::signature::SignatureBuilder::set_notation()
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use sequoia_openpgp as openpgp;
+    /// # use openpgp::Result;
+    /// use openpgp::cert::prelude::*;
+    /// use openpgp::packet::signature::subpacket::NotationDataFlags;
+    ///
+    /// # fn main() -> Result<()> {
+    /// let builder = CertRevocationBuilder::new().set_notation(
+    ///     "revocation-policy@example.org",
+    ///     "https://policy.example.org/cert-revocation-policy",
+    ///     NotationDataFlags::empty().set_human_readable(),
+    ///     false,
+    /// );
+    /// # Ok(())
+    /// # }
+    pub fn set_notation<N, V, F>(self, name: N, value: V, flags: F,
+                                 critical: bool)
+        -> Result<Self>
+    where
+        N: AsRef<str>,
+        V: AsRef<[u8]>,
+        F: Into<Option<NotationDataFlags>>,
+    {
+        Ok(Self {
+            builder: self.builder.set_notation(name, value, flags, critical)?
+        })
+    }
+
     /// Returns a signed revocation certificate.
     ///
     /// A revocation certificate is generated for `cert` and signed
@@ -249,7 +329,7 @@ impl TryFrom<signature::SignatureBuilder> for CertRevocationBuilder {
             return Err(
                 crate::Error::InvalidArgument(
                     format!("Expected signature type to be KeyRevocation but got {}",
-                            builder.typ()).into()).into());
+                            builder.typ())).into());
         }
         Ok(Self {
             builder
@@ -333,6 +413,7 @@ pub struct SubkeyRevocationBuilder {
 }
 assert_send_and_sync!(SubkeyRevocationBuilder);
 
+#[allow(clippy::new_without_default)]
 impl SubkeyRevocationBuilder {
     /// Returns a new `SubkeyRevocationBuilder`.
     ///
@@ -417,6 +498,84 @@ impl SubkeyRevocationBuilder {
         })
     }
 
+    /// Adds a notation to the revocation certificate.
+    ///
+    /// Unlike the [`SubkeyRevocationBuilder::set_notation`] method, this function
+    /// does not first remove any existing notation with the specified name.
+    ///
+    /// See [`SignatureBuilder::add_notation`] for further documentation.
+    ///
+    /// [`SignatureBuilder::add_notation`]: crate::packet::signature::SignatureBuilder::add_notation()
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use sequoia_openpgp as openpgp;
+    /// # use openpgp::Result;
+    /// use openpgp::cert::prelude::*;
+    /// use openpgp::packet::signature::subpacket::NotationDataFlags;
+    ///
+    /// # fn main() -> Result<()> {
+    /// let builder = CertRevocationBuilder::new().add_notation(
+    ///     "revocation-policy@example.org",
+    ///     "https://policy.example.org/cert-revocation-policy",
+    ///     NotationDataFlags::empty().set_human_readable(),
+    ///     false,
+    /// );
+    /// # Ok(())
+    /// # }
+    pub fn add_notation<N, V, F>(self, name: N, value: V, flags: F,
+                                 critical: bool)
+        -> Result<Self>
+    where
+        N: AsRef<str>,
+        V: AsRef<[u8]>,
+        F: Into<Option<NotationDataFlags>>,
+    {
+        Ok(Self {
+            builder: self.builder.add_notation(name, value, flags, critical)?
+        })
+    }
+
+    /// Sets a notation to the revocation certificate.
+    ///
+    /// Unlike the [`SubkeyRevocationBuilder::add_notation`] method, this function
+    /// first removes any existing notation with the specified name.
+    ///
+    /// See [`SignatureBuilder::set_notation`] for further documentation.
+    ///
+    /// [`SignatureBuilder::set_notation`]: crate::packet::signature::SignatureBuilder::set_notation()
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use sequoia_openpgp as openpgp;
+    /// # use openpgp::Result;
+    /// use openpgp::cert::prelude::*;
+    /// use openpgp::packet::signature::subpacket::NotationDataFlags;
+    ///
+    /// # fn main() -> Result<()> {
+    /// let builder = CertRevocationBuilder::new().set_notation(
+    ///     "revocation-policy@example.org",
+    ///     "https://policy.example.org/cert-revocation-policy",
+    ///     NotationDataFlags::empty().set_human_readable(),
+    ///     false,
+    /// );
+    /// # Ok(())
+    /// # }
+    pub fn set_notation<N, V, F>(self, name: N, value: V, flags: F,
+                                 critical: bool)
+        -> Result<Self>
+    where
+        N: AsRef<str>,
+        V: AsRef<[u8]>,
+        F: Into<Option<NotationDataFlags>>,
+    {
+        Ok(Self {
+            builder: self.builder.set_notation(name, value, flags, critical)?
+        })
+    }
+
     /// Returns a signed revocation certificate.
     ///
     /// A revocation certificate is generated for `cert` and `key` and
@@ -493,7 +652,7 @@ impl TryFrom<signature::SignatureBuilder> for SubkeyRevocationBuilder {
             return Err(
                 crate::Error::InvalidArgument(
                     format!("Expected signature type to be SubkeyRevocation but got {}",
-                            builder.typ()).into()).into());
+                            builder.typ())).into());
         }
         Ok(Self {
             builder
@@ -523,7 +682,7 @@ impl TryFrom<signature::SignatureBuilder> for SubkeyRevocationBuilder {
 /// Semantically, such a revocation certificate is currently
 /// meaningless.
 ///
-/// [User ID]: ../packet/struct.UserID.html
+/// [User ID]: crate::packet::UserID
 /// [designated revoker]: https://tools.ietf.org/html/rfc4880#section-5.2.3.15
 ///
 /// # Examples
@@ -579,6 +738,7 @@ pub struct UserIDRevocationBuilder {
 }
 assert_send_and_sync!(UserIDRevocationBuilder);
 
+#[allow(clippy::new_without_default)]
 impl UserIDRevocationBuilder {
     /// Returns a new `UserIDRevocationBuilder`.
     ///
@@ -607,8 +767,8 @@ impl UserIDRevocationBuilder {
     /// IDs.  This parameter is not fixed, however, to allow the use
     /// of the [private name space].
     ///
-    /// [`ReasonForRevocation::UIDRetired`]: ../types/enum.ReasonForRevocation.html#variant.UIDRetired
-    /// [private name space]: ../types/enum.ReasonForRevocation.html#variant.Private
+    /// [`ReasonForRevocation::UIDRetired`]: crate::types::ReasonForRevocation::UIDRetired
+    /// [private name space]: crate::types::ReasonForRevocation::Private
     ///
     ///
     /// # Examples
@@ -670,6 +830,84 @@ impl UserIDRevocationBuilder {
         })
     }
 
+    /// Adds a notation to the revocation certificate.
+    ///
+    /// Unlike the [`UserIDRevocationBuilder::set_notation`] method, this function
+    /// does not first remove any existing notation with the specified name.
+    ///
+    /// See [`SignatureBuilder::add_notation`] for further documentation.
+    ///
+    /// [`SignatureBuilder::add_notation`]: crate::packet::signature::SignatureBuilder::add_notation()
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use sequoia_openpgp as openpgp;
+    /// # use openpgp::Result;
+    /// use openpgp::cert::prelude::*;
+    /// use openpgp::packet::signature::subpacket::NotationDataFlags;
+    ///
+    /// # fn main() -> Result<()> {
+    /// let builder = CertRevocationBuilder::new().add_notation(
+    ///     "revocation-policy@example.org",
+    ///     "https://policy.example.org/cert-revocation-policy",
+    ///     NotationDataFlags::empty().set_human_readable(),
+    ///     false,
+    /// );
+    /// # Ok(())
+    /// # }
+    pub fn add_notation<N, V, F>(self, name: N, value: V, flags: F,
+                                 critical: bool)
+        -> Result<Self>
+    where
+        N: AsRef<str>,
+        V: AsRef<[u8]>,
+        F: Into<Option<NotationDataFlags>>,
+    {
+        Ok(Self {
+            builder: self.builder.add_notation(name, value, flags, critical)?
+        })
+    }
+
+    /// Sets a notation to the revocation certificate.
+    ///
+    /// Unlike the [`UserIDRevocationBuilder::add_notation`] method, this function
+    /// first removes any existing notation with the specified name.
+    ///
+    /// See [`SignatureBuilder::set_notation`] for further documentation.
+    ///
+    /// [`SignatureBuilder::set_notation`]: crate::packet::signature::SignatureBuilder::set_notation()
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use sequoia_openpgp as openpgp;
+    /// # use openpgp::Result;
+    /// use openpgp::cert::prelude::*;
+    /// use openpgp::packet::signature::subpacket::NotationDataFlags;
+    ///
+    /// # fn main() -> Result<()> {
+    /// let builder = CertRevocationBuilder::new().set_notation(
+    ///     "revocation-policy@example.org",
+    ///     "https://policy.example.org/cert-revocation-policy",
+    ///     NotationDataFlags::empty().set_human_readable(),
+    ///     false,
+    /// );
+    /// # Ok(())
+    /// # }
+    pub fn set_notation<N, V, F>(self, name: N, value: V, flags: F,
+                                 critical: bool)
+        -> Result<Self>
+    where
+        N: AsRef<str>,
+        V: AsRef<[u8]>,
+        F: Into<Option<NotationDataFlags>>,
+    {
+        Ok(Self {
+            builder: self.builder.set_notation(name, value, flags, critical)?
+        })
+    }
+
     /// Returns a signed revocation certificate.
     ///
     /// A revocation certificate is generated for `cert` and `userid`
@@ -745,7 +983,7 @@ impl TryFrom<signature::SignatureBuilder> for UserIDRevocationBuilder {
             return Err(
                 crate::Error::InvalidArgument(
                     format!("Expected signature type to be CertificationRevocation but got {}",
-                            builder.typ()).into()).into());
+                            builder.typ())).into());
         }
         Ok(Self {
             builder
@@ -776,7 +1014,7 @@ impl TryFrom<signature::SignatureBuilder> for UserIDRevocationBuilder {
 /// `A`.  Semantically, such a revocation certificate is currently
 /// meaningless.
 ///
-/// [User Attribute]: ../packet/user_attribute/index.html
+/// [User Attribute]: crate::packet::user_attribute
 /// [designated revoker]: https://tools.ietf.org/html/rfc4880#section-5.2.3.15
 ///
 /// # Examples
@@ -839,6 +1077,7 @@ pub struct UserAttributeRevocationBuilder {
 }
 assert_send_and_sync!(UserAttributeRevocationBuilder);
 
+#[allow(clippy::new_without_default)]
 impl UserAttributeRevocationBuilder {
     /// Returns a new `UserAttributeRevocationBuilder`.
     ///
@@ -867,8 +1106,8 @@ impl UserAttributeRevocationBuilder {
     /// Attributes.  This parameter is not fixed, however, to allow
     /// the use of the [private name space].
     ///
-    /// [`ReasonForRevocation::UIDRetired`]: ../types/enum.ReasonForRevocation.html#variant.UIDRetired
-    /// [private name space]: ../types/enum.ReasonForRevocation.html#variant.Private
+    /// [`ReasonForRevocation::UIDRetired`]: crate::types::ReasonForRevocation::UIDRetired
+    /// [private name space]: crate::types::ReasonForRevocation::Private
     ///
     /// # Examples
     ///
@@ -929,6 +1168,84 @@ impl UserAttributeRevocationBuilder {
         })
     }
 
+    /// Adds a notation to the revocation certificate.
+    ///
+    /// Unlike the [`UserAttributeRevocationBuilder::set_notation`] method, this function
+    /// does not first remove any existing notation with the specified name.
+    ///
+    /// See [`SignatureBuilder::add_notation`] for further documentation.
+    ///
+    /// [`SignatureBuilder::add_notation`]: crate::packet::signature::SignatureBuilder::add_notation()
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use sequoia_openpgp as openpgp;
+    /// # use openpgp::Result;
+    /// use openpgp::cert::prelude::*;
+    /// use openpgp::packet::signature::subpacket::NotationDataFlags;
+    ///
+    /// # fn main() -> Result<()> {
+    /// let builder = CertRevocationBuilder::new().add_notation(
+    ///     "revocation-policy@example.org",
+    ///     "https://policy.example.org/cert-revocation-policy",
+    ///     NotationDataFlags::empty().set_human_readable(),
+    ///     false,
+    /// );
+    /// # Ok(())
+    /// # }
+    pub fn add_notation<N, V, F>(self, name: N, value: V, flags: F,
+                                 critical: bool)
+        -> Result<Self>
+    where
+        N: AsRef<str>,
+        V: AsRef<[u8]>,
+        F: Into<Option<NotationDataFlags>>,
+    {
+        Ok(Self {
+            builder: self.builder.add_notation(name, value, flags, critical)?
+        })
+    }
+
+    /// Sets a notation to the revocation certificate.
+    ///
+    /// Unlike the [`UserAttributeRevocationBuilder::add_notation`] method, this function
+    /// first removes any existing notation with the specified name.
+    ///
+    /// See [`SignatureBuilder::set_notation`] for further documentation.
+    ///
+    /// [`SignatureBuilder::set_notation`]: crate::packet::signature::SignatureBuilder::set_notation()
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use sequoia_openpgp as openpgp;
+    /// # use openpgp::Result;
+    /// use openpgp::cert::prelude::*;
+    /// use openpgp::packet::signature::subpacket::NotationDataFlags;
+    ///
+    /// # fn main() -> Result<()> {
+    /// let builder = CertRevocationBuilder::new().set_notation(
+    ///     "revocation-policy@example.org",
+    ///     "https://policy.example.org/cert-revocation-policy",
+    ///     NotationDataFlags::empty().set_human_readable(),
+    ///     false,
+    /// );
+    /// # Ok(())
+    /// # }
+    pub fn set_notation<N, V, F>(self, name: N, value: V, flags: F,
+                                 critical: bool)
+        -> Result<Self>
+    where
+        N: AsRef<str>,
+        V: AsRef<[u8]>,
+        F: Into<Option<NotationDataFlags>>,
+    {
+        Ok(Self {
+            builder: self.builder.set_notation(name, value, flags, critical)?
+        })
+    }
+
     /// Returns a signed revocation certificate.
     ///
     /// A revocation certificate is generated for `cert` and `ua` and
@@ -1011,7 +1328,7 @@ impl TryFrom<signature::SignatureBuilder> for UserAttributeRevocationBuilder {
             return Err(
                 crate::Error::InvalidArgument(
                     format!("Expected signature type to be CertificationRevocation but got {}",
-                            builder.typ()).into()).into());
+                            builder.typ())).into());
         }
         Ok(Self {
             builder
