@@ -81,6 +81,7 @@ impl AEADAlgorithm {
             EAX => Ok(16),
             // According to RFC4880bis, Section 5.16.2.
             OCB => Ok(16),
+            GCM => Ok(16),
             _ => Err(Error::UnsupportedAEADAlgorithm(*self).into()),
         }
     }
@@ -95,6 +96,7 @@ impl AEADAlgorithm {
             // least 15 octets long".  GnuPG hardcodes 15 in
             // openpgp_aead_algo_info.
             OCB => Ok(15),
+            GCM => Ok(12),
             _ => Err(Error::UnsupportedAEADAlgorithm(*self).into()),
         }
     }
@@ -260,13 +262,13 @@ impl<'a, S: Schedule> Decryptor<'a, S> {
         -> Result<Self>
         where R: io::Read + Send + Sync + 'a
     {
-        Self::from_buffered_reader(
+        Self::from_cookie_reader(
             sym_algo, aead, chunk_size, schedule, key,
             Box::new(buffered_reader::Generic::with_cookie(
                 source, None, Default::default())))
     }
 
-    pub fn from_buffered_reader(sym_algo: SymmetricAlgorithm,
+    pub fn from_cookie_reader(sym_algo: SymmetricAlgorithm,
                             aead: AEADAlgorithm, chunk_size: usize,
                             schedule: S, key: SessionKey,
                             source: Box<dyn 'a + BufferedReader<Cookie>>)
@@ -486,7 +488,7 @@ impl<'a, S: Schedule> BufferedReaderDecryptor<'a, S> {
     {
         Ok(BufferedReaderDecryptor {
             reader: buffered_reader::Generic::with_cookie(
-                Decryptor::from_buffered_reader(
+                Decryptor::from_cookie_reader(
                     sym_algo, aead, chunk_size, schedule, key, source)?,
                 None, cookie),
         })
@@ -569,7 +571,7 @@ impl<'a, S: Schedule> BufferedReader<Cookie> for BufferedReaderDecryptor<'a, S>
 
     fn into_inner<'b>(self: Box<Self>)
             -> Option<Box<dyn BufferedReader<Cookie> + 'b>> where Self: 'b {
-        Some(self.reader.into_reader().source.as_boxed())
+        Some(self.reader.into_reader().source.into_boxed())
     }
 
     fn cookie_set(&mut self, cookie: Cookie) -> Cookie {
