@@ -7,7 +7,7 @@ use crate::packet::{key, Key};
 use crate::types::Curve;
 use crate::{Error, Result};
 
-use crate::crypto::ecdh::{encrypt_wrap, decrypt_unwrap};
+use crate::crypto::ecdh::{encrypt_wrap, decrypt_unwrap2};
 
 use win_crypto_ng as cng;
 use cng::asymmetric::{Ecdh, AsymmetricKey, Export};
@@ -153,6 +153,7 @@ pub fn decrypt<R>(
     recipient: &Key<key::PublicParts, R>,
     recipient_sec: &SecretKeyMaterial,
     ciphertext: &Ciphertext,
+    plaintext_len: Option<usize>,
 ) -> Result<SessionKey>
 where
     R: key::KeyRole,
@@ -284,5 +285,6 @@ where
         }
     };
 
-    decrypt_unwrap(recipient, &S, ciphertext)
+    decrypt_unwrap2(recipient.role_as_unspecified(), &S, ciphertext,
+                    plaintext_len)
 }
