@@ -4,7 +4,7 @@ use nettle::{curve25519, ecc, ecdh, random::Yarrow};
 
 use crate::{Error, Result};
 use crate::crypto::SessionKey;
-use crate::crypto::ecdh::{encrypt_wrap, decrypt_unwrap};
+use crate::crypto::ecdh::{encrypt_wrap, decrypt_unwrap2};
 use crate::crypto::mem::Protected;
 use crate::crypto::mpi::{MPI, PublicKey, SecretKeyMaterial, Ciphertext};
 use crate::packet::{key, Key};
@@ -119,7 +119,8 @@ pub fn encrypt<R>(recipient: &Key<key::PublicParts, R>,
 #[allow(non_snake_case)]
 pub fn decrypt<R>(recipient: &Key<key::PublicParts, R>,
                   recipient_sec: &SecretKeyMaterial,
-                  ciphertext: &Ciphertext)
+                  ciphertext: &Ciphertext,
+                  plaintext_len: Option<usize>)
     -> Result<SessionKey>
     where R: key::KeyRole
 {
@@ -213,7 +214,8 @@ pub fn decrypt<R>(recipient: &Key<key::PublicParts, R>,
                     Err(Error::UnsupportedEllipticCurve(curve.clone()).into()),
             };
 
-            decrypt_unwrap(recipient, &S, ciphertext)
+            decrypt_unwrap2(recipient.role_as_unspecified(), &S, ciphertext,
+                            plaintext_len)
         }
 
         _ =>
