@@ -2,12 +2,10 @@
 
 use std::convert::TryInto;
 
-use x25519_dalek_ng as x25519_dalek;
-
 use crate::{Error, Result};
 use crate::crypto::SessionKey;
 use crate::crypto::mem::Protected;
-use crate::crypto::ecdh::{encrypt_wrap, decrypt_unwrap};
+use crate::crypto::ecdh::{encrypt_wrap, decrypt_unwrap2};
 use crate::crypto::mpi::{self, Ciphertext, SecretKeyMaterial, MPI};
 use crate::packet::{key, Key};
 use crate::types::Curve;
@@ -40,7 +38,7 @@ pub fn encrypt<R>(recipient: &Key<key::PublicParts, R>,
             let recipient_key = PublicKey::from(R);
 
             // Generate a keypair and perform Diffie-Hellman.
-            let secret = EphemeralSecret::new(OsRng);
+            let secret = EphemeralSecret::random_from_rng(OsRng);
             let public = PublicKey::from(&secret);
             let shared = secret.diffie_hellman(&recipient_key);
 
@@ -89,7 +87,8 @@ pub fn encrypt<R>(recipient: &Key<key::PublicParts, R>,
 #[allow(non_snake_case)]
 pub fn decrypt<R>(recipient: &Key<key::PublicParts, R>,
                   recipient_sec: &SecretKeyMaterial,
-                  ciphertext: &Ciphertext)
+                  ciphertext: &Ciphertext,
+                  plaintext_len: Option<usize>)
     -> Result<SessionKey>
     where R: key::KeyRole
 {
@@ -144,5 +143,6 @@ pub fn decrypt<R>(recipient: &Key<key::PublicParts, R>,
         },
     };
 
-    decrypt_unwrap(recipient, &S, ciphertext)
+    decrypt_unwrap2(recipient.role_as_unspecified(), &S, ciphertext,
+                    plaintext_len)
 }
