@@ -20,6 +20,7 @@ use crate::packet::Key;
 use crate::types::{Curve, HashAlgorithm, PublicKeyAlgorithm, SymmetricAlgorithm};
 use crate::utils::{read_be_u64, write_be_u64};
 
+#[allow(unused_imports)]
 pub(crate) use crate::crypto::backend::ecdh::{encrypt, decrypt};
 
 /// Returns the default ECDH KDF hash function.
@@ -66,7 +67,7 @@ pub(crate) fn default_ecdh_kek_cipher(curve: &Curve) -> SymmetricAlgorithm {
 /// `VB` is the ephemeral public key encoded appropriately as MPI
 /// (i.e. with the 0x40 prefix for X25519, or 0x04 for the NIST
 /// curves), `S` is the shared Diffie-Hellman secret.
-#[allow(non_snake_case)]
+#[allow(non_snake_case, dead_code)]
 pub(crate) fn encrypt_wrap<R>(recipient: &Key<key::PublicParts, R>,
                               session_key: &SessionKey, VB: MPI,
                               S: &Protected)
@@ -116,11 +117,12 @@ pub(crate) fn encrypt_wrap<R>(recipient: &Key<key::PublicParts, R>,
 /// `recipient` is the message receiver's public key, `S` is the
 /// shared Diffie-Hellman secret used to encrypt `ciphertext`.
 #[allow(non_snake_case)]
-pub fn decrypt_unwrap<R>(recipient: &Key<key::PublicParts, R>,
-                         S: &Protected,
-                         ciphertext: &mpi::Ciphertext)
-    -> Result<SessionKey>
-    where R: key::KeyRole
+pub fn decrypt_unwrap2(recipient: &Key<key::PublicParts,
+                                       key::UnspecifiedRole>,
+                       S: &Protected,
+                       ciphertext: &mpi::Ciphertext,
+                       plaintext_len: Option<usize>)
+                       -> Result<SessionKey>
 {
     match (recipient.mpis(), ciphertext) {
         (mpi::PublicKey::ECDH { ref curve, ref hash, ref sym, ..},
@@ -135,8 +137,14 @@ pub fn decrypt_unwrap<R>(recipient: &Key<key::PublicParts, R>,
 
             // Compute m = AESKeyUnwrap( Z, C ) as per [RFC3394]
             let m = aes_key_unwrap(*sym, &Z, key)?;
-            let cipher = SymmetricAlgorithm::from(m[0]);
-            let m = pkcs5_unpad(m, 1 + cipher.key_size()? + 2)?;
+            let plaintext_len =
+                plaintext_len.ok_or_else(|| Error::InvalidOperation(
+                    "Need the plaintext length to decrypt this PKESK".into()))
+                .or_else(|_| -> Result<usize> {
+                    let cipher = SymmetricAlgorithm::from(m[0]);
+                    Ok(1 + cipher.key_size()? + 2)
+                })?;
+            let m = pkcs5_unpad(m, plaintext_len)?;
 
             Ok(m.into())
         },
@@ -147,6 +155,18 @@ pub fn decrypt_unwrap<R>(recipient: &Key<key::PublicParts, R>,
     }
 }
 
+/// Unwraps a session key.
+#[allow(non_snake_case)]
+#[deprecated(note = "Use decrypt_unwrap2")]
+pub fn decrypt_unwrap<R>(recipient: &Key<key::PublicParts, R>,
+                         S: &Protected,
+                         ciphertext: &mpi::Ciphertext)
+    -> Result<SessionKey>
+    where R: key::KeyRole
+{
+    decrypt_unwrap2(recipient.role_as_unspecified(), S, ciphertext, None)
+}
+
 /// Derives a secret key for session key wrapping.
 ///
 /// See [Section 7 of RFC 6637].
@@ -175,6 +195,7 @@ fn kdf(x: &Protected, obits: usize, hash: HashAlgorithm, param: &[u8])
 /// See [Section 8 of RFC 6637].
 ///
 ///   [Section 8 of RFC 6637]: https://tools.ietf.org/html/rfc6637#section-8
+#[allow(dead_code)]
 fn pkcs5_pad(sk: Protected, target_len: usize) -> Result<Protected> {
     if sk.len() > target_len {
         return Err(Error::InvalidArgument(
@@ -547,8 +568,10 @@ mod tests {
             const LAST: usize = 0;
 
             let s = s.as_ref();
-            assert_eq!(s[FIRST] & ! 0b1111_1000, 0);
-            assert_eq!(s[LAST] & 0b1100_0000, 0b0100_0000);
+            assert_eq!(s[FIRST] & ! 0b1111_1000, 0,
+                       "bits 0, 1 and 2 of the first byte should be cleared");
+            assert_eq!(s[LAST] & 0b1100_0000, 0b0100_0000,
+                       "bits 7 should be cleared and bit 6 should be set in the last byte");
         }
 
         for _ in 0..5 {
