@@ -1,5 +1,4 @@
 use std::convert::TryFrom;
-use std::cmp;
 use std::cmp::Ordering;
 use std::borrow::Borrow;
 
@@ -32,12 +31,22 @@ use crate::{
 ///   [RFC 4880]: https://tools.ietf.org/html/rfc4880
 ///   [Section 12.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-12.2
 ///
+/// # A Note on Equality
+///
+/// Like other data types, two `KeyHandle`s are considered equal if
+/// their serialized forms are the same.  That is, if you compare a
+/// key handle that contains a `Fingerprint`, and a key handle that
+/// contains a `KeyID`, they will not be considered equal **even if
+/// the key ID aliases the fingerprint**.  If you want to check for
+/// aliasing, you should use [`KeyHandle::aliases`].
+///
 /// # Examples
 ///
 /// ```rust
 /// # fn main() -> sequoia_openpgp::Result<()> {
 /// # use sequoia_openpgp as openpgp;
 /// use openpgp::KeyHandle;
+/// use openpgp::KeyID;
 /// use openpgp::Packet;
 /// use openpgp::parse::Parse;
 ///
@@ -59,12 +68,10 @@ use crate::{
 /// if let Packet::Signature(sig) = p {
 ///     let issuers = sig.get_issuers();
 ///     assert_eq!(issuers.len(), 2);
-///     assert_eq!(&issuers[0],
-///                &KeyHandle::Fingerprint(
-///                    "C03F A641 1B03 AE12 5764  6118 7223 B566 78E0 2528"
-///                        .parse()?));
-///     assert_eq!(&issuers[1],
-///                &KeyHandle::KeyID("7223 B566 78E0 2528".parse()?));
+///     let kh: KeyHandle
+///         = "C03F A641 1B03 AE12 5764  6118 7223 B566 78E0 2528".parse()?;
+///     assert!(&issuers[0].aliases(&kh));
+///     assert!(&issuers[1].aliases(&kh));
 /// } else {
 ///     unreachable!("It's a signature!");
 /// }
@@ -174,26 +181,7 @@ impl PartialOrd for KeyHandle {
     fn partial_cmp(&self, other: &KeyHandle) -> Option<Ordering> {
         let a = self.as_bytes();
         let b = other.as_bytes();
-
-        let l = cmp::min(a.len(), b.len());
-
-        // Do a little endian comparison so that for v4 keys (where
-        // the KeyID is a suffix of the Fingerprint) equivalent KeyIDs
-        // and Fingerprints sort next to each other.
-        for (a, b) in a[a.len()-l..].iter().zip(b[b.len()-l..].iter()) {
-            let cmp = a.cmp(b);
-            if cmp != Ordering::Equal {
-                return Some(cmp);
-            }
-        }
-
-        if a.len() == b.len() {
-            Some(Ordering::Equal)
-        } else {
-            // One (a KeyID) is the suffix of the other (a
-            // Fingerprint).
-            None
-        }
+        Some(a.cmp(b))
     }
 }
 
@@ -252,10 +240,6 @@ impl KeyHandle {
     /// fpr1 == keyid and fpr2 == keyid, but fpr1 != fpr2.
     /// ```
     ///
-    /// In these cases (and only these cases) `KeyHandle`'s
-    /// `PartialOrd` implementation returns `None` to correctly
-    /// indicate that a comparison is not possible.
-    ///
     /// This definition of equality makes searching for a given
     /// `KeyHandle` using `PartialEq` awkward.  This function fills
     /// that gap.  It answers the question: given two `KeyHandles`,
@@ -290,11 +274,32 @@ impl KeyHandle {
     pub fn aliases<H>(&self, other: H) -> bool
         where H: Borrow<KeyHandle>
     {
-        // This works, because the PartialOrd implementation only
-        // returns None if one value is a fingerprint and the other is
-        // a key id that matches the fingerprint's key id.
-        self.partial_cmp(other.borrow()).unwrap_or(Ordering::Equal)
-            == Ordering::Equal
+        let other = other.borrow();
+        if self.partial_cmp(other) == Some(Ordering::Equal) {
+            true
+        } else {
+            match (self, other) {
+                (KeyHandle::Fingerprint(Fingerprint::V4(f)),
+                 KeyHandle::KeyID(KeyID::V4(i)))
+                    | (KeyHandle::KeyID(KeyID::V4(i)),
+                       KeyHandle::Fingerprint(Fingerprint::V4(f))) =>
+                {
+                    // Avoid a heap allocation by embedding our
+                    // knowledge of how a v4 key ID is derived from a
+                    // v4 fingerprint:
+                    //
+                    // A v4 key ID are the 8 right-most octets of a v4
+                    // fingerprint.
+                    &f[12..] == i
+                },
+                (KeyHandle::Fingerprint(f), KeyHandle::KeyID(i))
+                    | (KeyHandle::KeyID(i), KeyHandle::Fingerprint(f)) =>
+                {
+                    &KeyID::from(f) == i
+                },
+                _ => false,
+            }
+        }
     }
 
     /// Returns whether the KeyHandle is invalid.
@@ -396,8 +401,19 @@ impl KeyHandle {
 
 #[cfg(test)]
 mod tests {
+    use quickcheck::{Arbitrary, Gen};
     use super::*;
 
+    impl Arbitrary for KeyHandle {
+        fn arbitrary(g: &mut Gen) -> Self {
+            if bool::arbitrary(g) {
+                Fingerprint::arbitrary(g).into()
+            } else {
+                KeyID::arbitrary(g).into()
+            }
+        }
+    }
+
     #[test]
     fn upper_hex_formatting() {
         let handle = KeyHandle::Fingerprint(Fingerprint::V4([1, 2, 3, 4, 5, 6, 7,
@@ -456,4 +472,34 @@ mod tests {
 
         Ok(())
     }
+
+    quickcheck! {
+        fn partial_cmp_is_asymmetric(a: KeyHandle, b: KeyHandle)
+                                     -> bool {
+            use Ordering::*;
+            true
+                && (! (a.partial_cmp(&b) == Some(Less))
+                    || ! (a.partial_cmp(&b) == Some(Greater)))
+                && (! (a.partial_cmp(&b) == Some(Greater))
+                    || ! (a.partial_cmp(&b) == Some(Less)))
+        }
+    }
+
+    quickcheck! {
+        fn partial_cmp_is_transitive(a: KeyHandle, b: KeyHandle, c: KeyHandle)
+                                     -> bool {
+            use Ordering::*;
+            true
+                && (! (a.partial_cmp(&b) == Some(Less)
+                       && b.partial_cmp(&c) == Some(Less))
+                    || a.partial_cmp(&c) == Some(Less))
+                && (! (a.partial_cmp(&b) == Some(Equal)
+                       && b.partial_cmp(&c) == Some(Equal))
+                    || a.partial_cmp(&c) == Some(Equal))
+                && (! (a.partial_cmp(&b) == Some(Greater)
+                       && b.partial_cmp(&c) == Some(Greater))
+                    || a.partial_cmp(&c) == Some(Greater))
+
+        }
+    }
 }
