@@ -97,11 +97,18 @@ impl std::str::FromStr for KeyID {
     type Err = anyhow::Error;
 
     fn from_str(s: &str) -> std::result::Result<Self, Self::Err> {
+        if s.chars().filter(|c| ! c.is_whitespace()).count() % 2 == 1 {
+            return Err(Error::InvalidArgument(
+                "Odd number of nibbles".into()).into());
+        }
+
         let bytes = crate::fmt::hex::decode_pretty(s)?;
 
         // A KeyID is exactly 8 bytes long.
         if bytes.len() == 8 {
             Ok(KeyID::from_bytes(&bytes[..]))
+        } else if bytes.len() == 4 {
+            Err(Error::ShortKeyID(s.to_string()).into())
         } else {
             // Maybe a fingerprint was given.  Try to parse it and
             // convert it to a KeyID.
@@ -451,8 +458,19 @@ mod test {
         "GB3751F1587DAEF1".parse::<KeyID>().unwrap_err();
         "EFB3751F1587DAEF1".parse::<KeyID>().unwrap_err();
         "%FB3751F1587DAEF1".parse::<KeyID>().unwrap_err();
-        assert_match!(KeyID::Invalid(_) = "587DAEF1".parse().unwrap());
-        assert_match!(KeyID::Invalid(_) = "0x587DAEF1".parse().unwrap());
+    }
+
+    #[test]
+    fn from_hex_short_keyid() {
+        for s in &[ "FB3751F1", "0xFB3751F1", "fb3751f1",  "0xfb3751f1" ] {
+            match s.parse::<KeyID>() {
+                Ok(_) => panic!("Failed to reject short Key ID."),
+                Err(err) => {
+                    let err = err.downcast_ref::<Error>().unwrap();
+                    assert!(matches!(err, Error::ShortKeyID(_)));
+                }
+            }
+        }
     }
 
     #[test]
