@@ -66,7 +66,7 @@ impl fmt::Debug for KeyServerPreferences {
             need_comma = true;
         }
 
-        for i in self.0.iter() {
+        for i in self.0.iter_set() {
             match i {
                 KEYSERVER_PREFERENCE_NO_MODIFY => (),
                 i => {
@@ -78,8 +78,7 @@ impl fmt::Debug for KeyServerPreferences {
         }
 
         // Mention any padding, as equality is sensitive to this.
-        let padding = self.0.padding_len();
-        if padding > 0 {
+        if let Some(padding) = self.0.padding_bytes() {
             if need_comma { f.write_str(", ")?; }
             write!(f, "+padding({} bytes)", padding)?;
         }
@@ -99,9 +98,14 @@ impl KeyServerPreferences {
         Self::new(&[])
     }
 
-    /// Returns a slice containing the raw values.
-    pub(crate) fn as_slice(&self) -> &[u8] {
-        self.0.as_slice()
+    /// Returns a reference to the underlying [`Bitfield`].
+    pub fn as_bitfield(&self) -> &Bitfield {
+        &self.0
+    }
+
+    /// Returns a mutable reference to the underlying [`Bitfield`].
+    pub fn as_bitfield_mut(&mut self) -> &mut Bitfield {
+        &mut self.0
     }
 
     /// Compares two key server preference sets for semantic equality.
@@ -175,8 +179,10 @@ impl KeyServerPreferences {
     /// # assert!(! ksp.no_modify());
     /// # Ok(()) }
     /// ```
-    pub fn set(self, bit: usize) -> Self {
-        Self(self.0.set(bit))
+    pub fn set(mut self, bit: usize) -> Self {
+        self.0.set(bit);
+        self.0.canonicalize();
+        self
     }
 
     /// Clears the specified keyserver preference flag.
@@ -199,8 +205,10 @@ impl KeyServerPreferences {
     /// # assert!(! ksp.no_modify());
     /// # Ok(()) }
     /// ```
-    pub fn clear(self, bit: usize) -> Self {
-        Self(self.0.clear(bit))
+    pub fn clear(mut self, bit: usize) -> Self {
+        self.0.clear(bit);
+        self.0.canonicalize();
+        self
     }
 
     /// Returns whether the certificate's owner requests that the
@@ -298,16 +306,19 @@ mod tests {
 
     quickcheck! {
         fn roundtrip(val: KeyServerPreferences) -> bool {
-            let mut q = KeyServerPreferences::new(val.as_slice());
+            let mut q_bytes = val.as_bitfield().as_bytes().to_vec();
+            let q = KeyServerPreferences::new(&q_bytes);
             assert_eq!(val, q);
             assert!(val.normalized_eq(&q));
 
             // Add some padding to q.  Make sure they are still equal.
-            q.0.raw.push(0);
+            q_bytes.push(0);
+            let q = KeyServerPreferences::new(&q_bytes);
             assert!(val != q);
             assert!(val.normalized_eq(&q));
 
-            q.0.raw.push(0);
+            q_bytes.push(0);
+            let q = KeyServerPreferences::new(&q_bytes);
             assert!(val != q);
             assert!(val.normalized_eq(&q));
 
