@@ -26,9 +26,9 @@ use crate::{
 /// Signature creation time is internally stored as a `Timestamp`:
 ///
 /// Note that this example retrieves raw packet value.
-/// Use [`SubpacketArea::signature_creation_time`] to get the signature creation time.
+/// Use [`SubpacketAreas::signature_creation_time`] to get the signature creation time.
 ///
-/// [`SubpacketArea::signature_creation_time`]: ../packet/signature/subpacket/struct.SubpacketArea.html#method.signature_creation_time
+/// [`SubpacketAreas::signature_creation_time`]: crate::packet::signature::subpacket::SubpacketAreas::signature_creation_time()
 ///
 /// ```
 /// use sequoia_openpgp as openpgp;
@@ -60,7 +60,7 @@ use crate::{
 ///
 /// [Section 3.5 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-3.5
 /// [UNIX epoch]: https://en.wikipedia.org/wiki/Unix_time
-/// [`Timestamp::round_down`]: ../types/struct.Timestamp.html#method.round_down
+/// [`Timestamp::round_down`]: crate::types::Timestamp::round_down()
 #[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Hash)]
 pub struct Timestamp(u32);
 assert_send_and_sync!(Timestamp);
@@ -124,7 +124,7 @@ impl fmt::Debug for Timestamp {
 impl Timestamp {
     /// Returns the current time.
     pub fn now() -> Timestamp {
-        SystemTime::now().try_into()
+        crate::now().try_into()
             .expect("representable for the next hundred years")
     }
 
@@ -133,7 +133,7 @@ impl Timestamp {
     /// Returns `None` if the resulting timestamp is not
     /// representable.
     pub fn checked_add(&self, d: Duration) -> Option<Timestamp> {
-        self.0.checked_add(d.0).map(|v| Self(v))
+        self.0.checked_add(d.0).map(Self)
     }
 
     /// Subtracts a duration from this timestamp.
@@ -141,7 +141,7 @@ impl Timestamp {
     /// Returns `None` if the resulting timestamp is not
     /// representable.
     pub fn checked_sub(&self, d: Duration) -> Option<Timestamp> {
-        self.0.checked_sub(d.0).map(|v| Self(v))
+        self.0.checked_sub(d.0).map(Self)
     }
 
     /// Rounds down to the given level of precision.
@@ -161,7 +161,7 @@ impl Timestamp {
     /// The lower limit `floor` represents the earliest time the timestamp will be
     /// rounded down to.
     ///
-    /// See also [`Duration::round_up`](struct.Duration.html#method.round_up).
+    /// See also [`Duration::round_up`](Duration::round_up()).
     ///
     /// # Important note
     ///
@@ -252,7 +252,7 @@ impl Timestamp {
 
 #[cfg(test)]
 impl Arbitrary for Timestamp {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         Timestamp(u32::arbitrary(g))
     }
 }
@@ -341,44 +341,57 @@ impl Duration {
     /// Returns a `Duration` with the given number of minutes, if
     /// representable.
     pub fn minutes(n: u32) -> Result<Duration> {
-        60u32.checked_mul(n).ok_or(())
-            .map(Self::seconds)
-            .map_err(|_| Error::InvalidArgument(
-                format!("Not representable: {} minutes in seconds exceeds u32",
-                        n)).into())
+        match 60u32.checked_mul(n) {
+            Some(val) => Ok(Self::seconds(val)),
+            None => {
+                Err(Error::InvalidArgument(format!(
+                    "Not representable: {} minutes in seconds exceeds u32",
+                    n
+                )).into())
+            }
+        }
     }
 
     /// Returns a `Duration` with the given number of hours, if
     /// representable.
     pub fn hours(n: u32) -> Result<Duration> {
-        60u32.checked_mul(n)
-            .ok_or(Error::InvalidArgument("".into()).into())
-            .and_then(Self::minutes)
-            .map_err(|_| Error::InvalidArgument(
-                format!("Not representable: {} hours in seconds exceeds u32",
-                        n)).into())
+        match 60u32.checked_mul(n) {
+            Some(val) => Self::minutes(val),
+            None => {
+                Err(Error::InvalidArgument(format!(
+                    "Not representable: {} hours in seconds exceeds u32",
+                    n
+                )).into())
+            }
+        }
     }
 
     /// Returns a `Duration` with the given number of days, if
     /// representable.
     pub fn days(n: u32) -> Result<Duration> {
-        24u32.checked_mul(n)
-            .ok_or(Error::InvalidArgument("".into()).into())
-            .and_then(Self::hours)
-            .map_err(|_| Error::InvalidArgument(
-                format!("Not representable: {} days in seconds exceeds u32",
-                        n)).into())
+        match 24u32.checked_mul(n) {
+            Some(val) => Self::hours(val),
+            None => {
+                Err(Error::InvalidArgument(format!(
+                    "Not representable: {} days in seconds exceeds u32",
+                    n
+                )).into())
+            }
+        }
     }
 
     /// Returns a `Duration` with the given number of weeks, if
     /// representable.
     pub fn weeks(n: u32) -> Result<Duration> {
-        7u32.checked_mul(n)
-            .ok_or(Error::InvalidArgument("".into()).into())
-            .and_then(Self::days)
-            .map_err(|_| Error::InvalidArgument(
-                format!("Not representable: {} weeks in seconds exceeds u32",
-                        n)).into())
+        match 7u32.checked_mul(n) {
+            Some(val) => Self::days(val),
+            None => {
+                Err(Error::InvalidArgument(format!(
+                    "Not representable: {} weeks in seconds exceeds u32",
+                    n
+                )).into())
+            }
+        }
     }
 
     /// Returns a `Duration` with the given number of years, if
@@ -415,7 +428,7 @@ impl Duration {
     /// while avoiding the metadata leak associated with preserving
     /// the originally intended expiration time.
     ///
-    ///   [`Timestamp::round_down`]: struct.Timestamp.html#method.round_down
+    ///   [`Timestamp::round_down`]: Timestamp::round_down()
     ///
     /// The given level `p` determines the resulting resolution of
     /// `2^p` seconds.  The default is `21`, which results in a
@@ -595,7 +608,7 @@ impl Timestamp {
 
 #[cfg(test)]
 impl Arbitrary for Duration {
-    fn arbitrary<G: Gen>(g: &mut G) -> Self {
+    fn arbitrary(g: &mut Gen) -> Self {
         Duration(u32::arbitrary(g))
     }
 }
@@ -642,7 +655,9 @@ mod tests {
         fn duration_round_up(d: Duration) -> bool {
             let u = d.round_up(None, None).unwrap();
             assert!(d <= u);
-            assert_eq!(u32::from(u) & 0b1_1111_1111_1111_1111_1111, 0);
+            assert!(u32::from(u) & 0b1_1111_1111_1111_1111_1111 == 0
+                || u32::from(u) == u32::MAX
+            );
             assert!(u32::from(u) - u32::from(d) < 2_u32.pow(21));
             true
         }
@@ -669,21 +684,46 @@ mod tests {
     }
 
     // #668
-    // Ensure that, on x86, Timestamps between i32::MAX + 1 and u32::MAX are
-    // clamped down to i32::MAX, and values below are not altered.
-    #[cfg(target_arch = "x86")]
+    // Ensure that, on systems where the SystemTime can only represent values
+    // up to i32::MAX (generally, 32-bit systems), Timestamps between
+    // i32::MAX + 1 and u32::MAX are clamped down to i32::MAX, and values below
+    // are not altered.
     #[test]
     fn system_time_32_bit() -> Result<()> {
-        let t1 = Timestamp::from(u32::MAX);
-        let t2 = Timestamp::from(i32::MAX as u32 + 1);
-        assert_eq!(SystemTime::from(t1),
-                   UNIX_EPOCH + SystemDuration::new(i32::MAX as u64, 0));
-        assert_eq!(SystemTime::from(t2),
-                   UNIX_EPOCH + SystemDuration::new(i32::MAX as u64, 0));
+        let is_system_time_too_small = UNIX_EPOCH
+            .checked_add(SystemDuration::new(i32::MAX as u64 + 1, 0))
+            .is_none();
 
-        let t3 = Timestamp::from(i32::MAX as u32 - 1);
-        assert!(SystemTime::from(t3),
-                UNIX_EPOCH + SystemDuration::new(i32::MAX as u64 - 1, 0));
+        let t1 = Timestamp::from(i32::MAX as u32 - 1);
+        let t2 = Timestamp::from(i32::MAX as u32);
+        let t3 = Timestamp::from(i32::MAX as u32 + 1);
+        let t4 = Timestamp::from(u32::MAX);
+
+        if is_system_time_too_small {
+          assert_eq!(SystemTime::from(t1),
+                     UNIX_EPOCH + SystemDuration::new(i32::MAX as u64 - 1, 0));
+
+          assert_eq!(SystemTime::from(t2),
+                     UNIX_EPOCH + SystemDuration::new(i32::MAX as u64, 0));
+
+          assert_eq!(SystemTime::from(t3),
+                     UNIX_EPOCH + SystemDuration::new(i32::MAX as u64, 0));
+
+          assert_eq!(SystemTime::from(t4),
+                     UNIX_EPOCH + SystemDuration::new(i32::MAX as u64, 0));
+        } else {
+          assert_eq!(SystemTime::from(t1),
+                     UNIX_EPOCH + SystemDuration::new(i32::MAX as u64 - 1, 0));
+
+          assert_eq!(SystemTime::from(t2),
+                     UNIX_EPOCH + SystemDuration::new(i32::MAX as u64, 0));
+
+          assert_eq!(SystemTime::from(t3),
+                     UNIX_EPOCH + SystemDuration::new(i32::MAX as u64 + 1, 0));
+
+          assert_eq!(SystemTime::from(t4),
+                     UNIX_EPOCH + SystemDuration::new(u32::MAX as u64, 0));
+        }
         Ok(())
     }
 }
